//Maxim Integrated Products DS1WM API Revision 1.2

//INCLUDES
#include <p18f4550.h>
#include "DS1WM.h"
#include "delays.h"

//DEFINES
#define BYTE unsigned char

//VARIABLES
BYTE InterruptRegisterData;
BYTE ReceiveBufferData;
BYTE CommandRegisterData;
int ErrorStatus;

//Search ROM Variables
BYTE LastDiscrepancy = -1;
BYTE CurrentDiscrepancy = -1;

int ROMCRCCheck(BYTE ROMCodes[][8], BYTE DeviceNumber){
	BYTE Output1,Output2,Output3,Output4,Output5,Output6,Output7,Output8;
	BYTE X0,X1,X2,X3,X4,X5,X6,X7,X8;
	BYTE DataInput=0;
	BYTE CRC=0;
	BYTE ExpectedCRC=0;
	int i,j;

	ExpectedCRC = ROMCodes[DeviceNumber][7];

	//Initialize Variable
	Output1 = 0;
	Output2 = 0;
	Output3 = 0;
	Output4 = 0;
	Output5 = 0;
	Output6 = 0;
	Output7 = 0;
	Output8 = 0;
	X0 = 0;
	X1 = 0;
	X2 = 0;
	X3 = 0;
	X4 = 0;
	X5 = 0;
	X6 = 0;
	X7 = 0;
	X8 = 0;

	for(i=0;i<=6;i++){		//7 Bytes
		for(j=0;j<=7;j++){	//8 Bits
			//Shift values in
	        Output1 = X0;
	        Output2 = X1;
	        Output3 = X2;
	        Output4 = X3;
	        Output5 = X4;
	        Output6 = X5;
	        Output7 = X6;
	        Output8 = X7;

	        //Calculate new values
			if((ROMCodes[DeviceNumber][i] >> j) & 0x01)
				DataInput = 1;
			else
				DataInput = 0;

	        X0 = DataInput ^ Output8;
	        X1 = Output1;
	        X2 = Output2;
	        X3 = Output3;
	        X4 = X0 ^ Output4;
	        X5 = X0 ^ Output5;
	        X6 = Output6;
	        X7 = Output7;
	        X8 = Output8;
		}
	}
	CRC = (X0 << 7) | (X1 << 6) | (X2 << 5) | (X3 << 4) | (X4 << 3) | (X5 << 2) | (X6 << 1) | X7;
	if(CRC == ExpectedCRC)
		return 1;
	else
		return 0;
}

int OWSearch(BYTE ROMCodes[][8]){
	//This function issues a 1-Wire reset followed by the Search ROM Command. The function will return any
	//errors associated with the OWReset function as well as the following:
	//* Error Status = -9 indicates that ROMCodes array passed is not sized large enough for the number of devices found.
	//	To fix this problem, modify MaxNumberDevices (DS1WM.h - default is 20 devices).
	//* Return 0 denotes an error (see ErrorStatus) for specific error.
	//* Return # where # is greater than zero. This number is the number of 1-Wire devices found.

	BYTE TransmitData[16]={0};
	BYTE ReceiveData[16]={0};
	BYTE ROMIndex=0;				//Index used to store in ROMCodes array.
	int i,j=0;

	//Initialize Array
	for(i=0;i<MaxNumberDevices;i++){
		for(j=0;j<8;j++){
			ROMCodes[i][j] = 0;
		}
	}

	while(1){
		if(!OWReset()){
			return 0;	//Return Error
		}

		OWWriteByte(0xF0);		//Send Search ROM Command
		CommandRegisterWrite(0x02);	//Set SRA bit (Search ROM Accelerator)

		for(i=0;i<16;i++){
			ReceiveData[i] = OWWriteByte(TransmitData[i]);	//Send Data (Rn Xn)

			//Updata transmit data. This data will be further modified once the highest
			//discrepancy is found.
			TransmitData[i] = ReceiveData[i];	//(Rn Dn)
		}

		j=0;	//Point to first byte in ReceiveData

		for(i=0;i<8;i++){
			//Extract ROM Code from Receieved Data (De-Interleave)
			//It takes two bytes from the ReceiveData to reconstruct one byte of the ROM ID.
			//Data Byte is in the form of:
			//r7 d7 r6 d6 r5 d5 r4 d4 r3 d3 r2 d2 r1 d1 r0 d0
			//Where we are only interested in rn (d's are discrepancies), where r0-r7 make up the byte.

			//Constructing LSNibble of Byte
			ROMCodes[ROMIndex][i] = (ReceiveData[j] >> 1) & 0x01;	//Bit0
			ROMCodes[ROMIndex][i] |= (ReceiveData[j] >> 2) & 0x02;	//Bit1
			ROMCodes[ROMIndex][i] |= (ReceiveData[j] >> 3) & 0x04;	//Bit2
			ROMCodes[ROMIndex][i] |= (ReceiveData[j] >> 4) & 0x08;	//Bit3

			j++;	//Point to next byte in ReceiveData

			//Constructing MSNibble of Byte
			ROMCodes[ROMIndex][i] |= (ReceiveData[j] << 3) & 0x10;	//Bit4
			ROMCodes[ROMIndex][i] |= (ReceiveData[j] << 2) & 0x20;	//Bit5
			ROMCodes[ROMIndex][i] |= (ReceiveData[j] << 1) & 0x40;	//Bit6
			ROMCodes[ROMIndex][i] |= ReceiveData[j] & 0x80;			//Bit7

			j++;	//Point to next byte in ReceiveData
		}

		if(!ROMCRCCheck(ROMCodes,ROMIndex)){
			ErrorStatus = -9;
			return 0;
		}

		//Find HIGHEST discrepancy (Start check at MSByte) and set rn at highest discrepancy
		//and clear all Rn higher than last discrepancy.
		for(i=15;i>=0;i--){		//Select Receive Data Byte
			if(ReceiveData[i] & 0x40){	//Check Bit 6 Discrepancy
				CurrentDiscrepancy = ((i+1)*4) - 1;

				//Check to see if Rn is already set, if set, clear Rn and continue looking for
				//next higher discrepancy. If Rn is set, we have already gone down both the zero and the
				//one branch.
				if(ReceiveData[i] & 0x80){
					//Clear Rn to 0 since we have already gone down both the 0 & 1 paths. We are still searching
					//for the highest discrepancy.
					TransmitData[i] = TransmitData[i] & 0x3F;	//Clear Bit 7 Rn/Dn and higher (Dn is a don't care)
				}else{
					//Set Rn to 1 at discrepancy location and all higher values to 0.
					TransmitData[i] = TransmitData[i] | 0x80;	//Set Bit 7 Rn
					break;
				}
			}

			if(ReceiveData[i] & 0x10){	//Check Bit 4 Discrepancy
				CurrentDiscrepancy = ((i+1)*4) - 2;

				//Check to see if Rn is already set, if set, clear Rn and continue looking for
				//next higher discrepancy. If Rn is set, we have already gone down both the zero and the
				//one branch.
				if(ReceiveData[i] & 0x20){
					//Clear Rn to 0 since we have already gone down both the 0 & 1 paths. We are still searching
					//for the highest discrepancy.
					TransmitData[i] = TransmitData[i] & 0x0F;	//Clear Bit 5 Rn/Dn and higher (Dn is a don't care)
				}else{
					//Set Rn to 1 at discrepancy location and all higher values to 0.
					TransmitData[i] = TransmitData[i] | 0x20;	//Set Bit 5 Rn
					TransmitData[i] = TransmitData[i] & 0x3F;	//Clear all Rn higher than last discrepancy
					break;
				}
			}

			if(ReceiveData[i] & 0x04){	//Check Bit 2 Discrepancy
				CurrentDiscrepancy = ((i+1)*4) - 3;

				//Check to see if Rn is already set, if set, clear Rn and continue looking for
				//next higher discrepancy. If Rn is set, we have already gone down both the zero and the
				//one branch.
				if(ReceiveData[i] & 0x08){
					//Clear Rn to 0 since we have already gone down both the 0 & 1 paths. We are still searching
					//for the highest discrepancy.
					TransmitData[i] = TransmitData[i] & 0x03;	//Clear Bit 3 Rn/Dn and higher (Dn is a don't care)
				}else{
					//Set Rn to 1 at discrepancy location and all higher values to 0.
					TransmitData[i] = TransmitData[i] | 0x08;	//Set Bit 3 Rn
					TransmitData[i] = TransmitData[i] & 0x0F;	//Clear all Rn higher than last discrepancy
					break;
				}
			}

			if(ReceiveData[i] & 0x01){	//Check Bit 1 Discrepancy
				CurrentDiscrepancy = ((i+1)*4) - 4;

				//Check to see if Rn is already set, if set, clear Rn and continue looking for
				//next higher discrepancy. If Rn is set, we have already gone down both the zero and the
				//one branch.
				if(ReceiveData[i] & 0x02){
					//Clear Rn to 0 since we have already gone down both the 0 & 1 paths. We are still searching
					//for the highest discrepancy.
					TransmitData[i] = TransmitData[i] & 0x00;	//Clear Bit 1 Rn/Dn and higher (Dn is a don't care)
				}else{
					//Set Rn to 1 at discrepancy location and all higher values to 0.
					TransmitData[i] = TransmitData[i] | 0x02;	//Set Bit 1 Rn
					TransmitData[i] = TransmitData[i] & 0x03;	//Clear all Rn higher than last discrepancy
					break;
				}
			}

			//No discrepancy found, clear entire byte
			TransmitData[i] = 0x00;
		}

		if(CurrentDiscrepancy != LastDiscrepancy && i != -1){
			LastDiscrepancy = CurrentDiscrepancy;
			if(ROMIndex == MaxNumberDevices-1){
				CommandRegisterWrite(0x00);	//Disable Search ROM Accelerator
				ErrorStatus = -10;
				return 0;	//ROMCodes array is not sized correctly.
			}
			else{
				ROMIndex++;
			}
		}
		else{	//Search is done.
			CommandRegisterWrite(0x00);	//Disable Search ROM Accelerator
			return ROMIndex + 1;	//Return number of devices found.
		}
	}
}

int MatchROM(BYTE ROMCodes[][8], BYTE DeviceNumber){
	int i;

	for(i=0;i<=8;i++){
		//Note: The transmit & receive buffers should be clear coming into this function.
		do{	//To prevent infinite loops, use a watchdog timer.
			//Check Transmit Buffer Empty (TBE) flag to know when we can send the byte.
			InterruptRegisterData = InterruptRegisterRead() & TBE;
		}while(!InterruptRegisterData);	//Exit Loop when TBE is set.

		//Send Data to transmit buffer, which then goes out on the 1-Wire IO
		TransmitBufferWrite(ROMCodes[DeviceNumber][i]);

		do{	//To prevent infinite loops, use a watchdog timer.
			//Check Receive Buffer Full (RBF) flag to know when we can read the data out.
			InterruptRegisterData = InterruptRegisterRead() & RBF;
		}while(!InterruptRegisterData);	//Exit Loop when RBF is set.

		ReceiveBufferData = ReceiveBufferRead();
		if(ROMCodes[DeviceNumber][i] != ReceiveBufferData){
			return i;	//Return location where a data mismatch occurred.
		}
	}
	return i;
}

BYTE OWReadByte(){
	//Note: The transmit & receive buffers should be clear coming into this function.
	do{	//To prevent infinite loops, use a watchdog timer.
		//Check Transmit Buffer Empty (TBE) flag to know when we can send the byte.
		InterruptRegisterData = InterruptRegisterRead() & TBE;
	}while(!InterruptRegisterData);	//Exit Loop when TBE is set.

	//Write FFh to the transmit buffer, which is wire AND'ed to the device data read back.
	//The result is then stored in the Receive Buffer.
	TransmitBufferWrite(0xFF);

	do{	//To prevent infinite loops, use a watchdog timer.
		//Check Receive Buffer Full (RBF) flag to know when we can read the data out.
		InterruptRegisterData = InterruptRegisterRead() & RBF;
	}while(!InterruptRegisterData);	//Exit Loop when RBF is set.

	ReceiveBufferData = ReceiveBufferRead();

	return ReceiveBufferData;
}

void OWReadBytes(BYTE *ReadDataArray, int BytesToRead){
	int i;

	//Note: The transmit & receive buffers should be clear coming into this function.
	for(i=0;i<BytesToRead;i++){
		do{	//To prevent infinite loops, use a watchdog timer.
			//Check Transmit Buffer Empty (TBE) flag to know when we can send the byte.
			InterruptRegisterData = InterruptRegisterRead() & TBE;
		}while(!InterruptRegisterData);	//Exit Loop when TBE is set.

		//Write FFh to the transmit buffer, which is wire AND'ed to the device data read back.
		//The result is then stored in the Receive Buffer.
		TransmitBufferWrite(0xFF);

		do{	//To prevent infinite loops, use a watchdog timer.
			//Check Receive Buffer Full (RBF) flag to know when we can read the data out.
			InterruptRegisterData = InterruptRegisterRead() & RBF;
		}while(!InterruptRegisterData);	//Exit Loop when RBF is set.

		ReadDataArray[i] = ReceiveBufferRead();
	}
}

BYTE OWWriteByte(BYTE WriteByte){
	do{	//To prevent infinite loops, use a watchdog timer.
		//Check Transmit Buffer Empty (TBE) flag to know when we can send the byte.
		InterruptRegisterData = InterruptRegisterRead() & TBE;
	}while(!InterruptRegisterData);	//Exit Loop when TBE is set.

	//Send Data to transmit buffer, which then goes out on the 1-Wire IO
	TransmitBufferWrite(WriteByte);

	do{	//To prevent infinite loops, use a watchdog timer.
		//Check Receive Buffer Full (RBF) flag to know when we can read the data out.
		InterruptRegisterData = InterruptRegisterRead() & RBF;
	}while(!InterruptRegisterData);	//Exit Loop when RBF is set.

	ReceiveBufferData = ReceiveBufferRead();
	return ReceiveBufferData;
}

BYTE OWWriteBytes(BYTE *WriteDataArray, int BytesToWrite){
	int i;

	//Note: The transmit & receive buffers should be clear coming into this function.
	for(i=0;i<BytesToWrite;i++){
		//
		do{	//To prevent infinite loops, use a watchdog timer.
			//Check Transmit Buffer Empty (TBE) flag to know when we can send the next byte.
			InterruptRegisterData = InterruptRegisterRead() & TBE;
		}while(!InterruptRegisterData);	//Exit Loop when TBE is set.

		//Send Data to transmit buffer, which then goes out on the 1-Wire IO
		TransmitBufferWrite(WriteDataArray[i]);

		//Note: The data being transmitted is AND'ed and stored in the receive buffer for
		//verification.
		do{	//To prevent infinite loops, use a watchdog timer.
			//Check Receive Buffer Full (RBF) flag to know when we can read the data out.
			InterruptRegisterData = InterruptRegisterRead() & RBF;
		}while(!InterruptRegisterData);	//Exit Loop when RBF is set.

		//Verify the data written via the Receive Buffer
		ReceiveBufferData = ReceiveBufferRead();
		if(WriteDataArray[i] != ReceiveBufferData){
			return i;	//Return location where a data mismatch occurred.
		}
	}
	return i;
}

int OWReset(){
	//OWReset: This function initiates a 1-Wire Reset Command through
	//the Command Register then polls the 1WR bit (OneWR) in the Command Register
	//until it is cleared. The Interrupt Register is read and checked for errors
	//being registered any of the applicable bits (OW_LOW, OW_SHORT, PDR & PD).
	//If an error is encountered, the function sets the ErrorStatus with the bit
	//position of the error (negative #), where the LSBit = 1, not zero. The function
	//will then return a zero. If a device is found, a 1 is returned. Please note that
	//the PDR register will show a 0 (Slave Found) if the 1-Wire line is shorted during
	//a 1-Wire Reset. Therefore, it is important to check all of the bits first
	//and check the PDR bit last.

	//Set 1WR bit in Control Register to initiate a 1-Wire Reset Pulse
	CommandRegisterWrite(OneWR);

	do{	//To prevent infinite loops, use a watchdog timer.
		CommandRegisterData = CommandRegisterRead() & OneWR;
	}while(CommandRegisterData);	//Exit Loop when 1WR(OneWR) is cleared.

	InterruptRegisterData = InterruptRegisterRead();

	//Verify the DS1WM accepted the Reset Command.
	if(!(InterruptRegisterData & PD)){
		ErrorStatus = -1;	//Return Error Code (Bit Position 1)
		return 0;
	}

	//Verify the IO was not shorted.
	if(InterruptRegisterData & OW_LOW){
		ErrorStatus = -8;	//Return Error Code (Bit Position 8)
		return 0;
	}

	//Verify the IO was not shorted.
	if(InterruptRegisterData & OW_SHORT){
		ErrorStatus = -7;	//Return Error Code (Bit Position 7)
		return 0;
	}

	//No Slave found.
	if(InterruptRegisterData & PDR){
		ErrorStatus = -2;	//Return Error Code (Bit Position 2)
		return 0;
	}

	//A device was found, return 1.
	return 1;
}

void SetClockFrequency(BYTE ClockSetting){
	//NOTE: Currently setup for 16MHz ONLY!
	ClockDivisorRegisterWrite(0x90);
}

void CommandRegisterWrite(BYTE WriteByte){
	WriteByte2Register(CommandRegister,WriteByte);
}

BYTE CommandRegisterRead(){
	return ReadByteFromRegister(CommandRegister);
}

void TransmitBufferWrite(BYTE WriteByte){
	WriteByte2Register(TransmitBufferRegister,WriteByte);
}

BYTE ReceiveBufferRead(){
	return ReadByteFromRegister(TransmitBufferRegister);
}

BYTE InterruptRegisterRead(){
	return ReadByteFromRegister(InterruptRegister);
}

void InterruptEnableRegisterWrite(BYTE WriteByte){
	WriteByte2Register(InterruptEnableRegister,WriteByte);
}

BYTE InterruptEnableRegisterRead(){
	return ReadByteFromRegister(InterruptEnableRegister);
}

void ClockDivisorRegisterWrite(BYTE WriteByte){
	WriteByte2Register(ClockDivisorRegister,WriteByte);
}

BYTE ClockDivisorRegisterRead(){
	return ReadByteFromRegister(ClockDivisorRegister);
}

void ControlRegisterWrite(BYTE WriteByte){
	WriteByte2Register(ControlRegister,WriteByte);
}

BYTE ControlRegisterRead(){
	return ReadByteFromRegister(ControlRegister);
}

BYTE ReadByteFromRegister(BYTE Address){
	//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	//NOTE: Only the lower three bits are used for addressing.
	//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	BYTE DataByteRead;

	TRISD = 0xFF;			//Ensure pins are set as INPUTS.

	ADS_BAR = 0;

	if(Address & 0x01){
		A0 = 1;
	}
	else{
		A0 = 0;
	}

	if(Address & 0x02){
		A1 = 1;
	}
	else{
		A1 = 0;
	}

	if(Address & 0x04){
		A2 = 1;
	}
	else{
		A2 = 0;
	}
	Delay100TCYx(12);	//Address Settling time

	EN_BAR = 0;
	Delay1TCY();			//Delay of 83.33nS (tES)

	ADS_BAR = 1;
	Delay1TCY();			//Delay of 83.33nS (tAR)

	RD_BAR = 0;
	Delay1TCY();			//Delay of 83.33nS (tWR)

	DataByteRead = DataBus;	//Read Data Lines
	Delay1TCY();			//Delay of 83.33nS (tWR)

	RD_BAR = 1;
	Delay1TCY();			//Delay of 83.33nS

	EN_BAR = 1;

	return DataByteRead;
}

void WriteByte2Register(BYTE Address, BYTE WriteByte){
	//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	//NOTE: Only the lower three bits are used for addressing.
	//@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
	TRISD = 0x00;			//Ensure pins are set as OUTPUTS.

	ADS_BAR = 0;			//Pull ADS line low to begin Write Cycle.

	//Set Address Lines
	if(Address & 0x01){
		A0 = 1;
	}
	else{
		A0 = 0;
	}

	if(Address & 0x02){
		A1 = 1;
	}
	else{
		A1 = 0;
	}

	if(Address & 0x04){
		A2 = 1;
	}
	else{
		A2 = 0;
	}

	EN_BAR = 0;
	Delay1TCY();			//Delay of 83.33nS (tES)

	//LATCH IN ADDRESS
	ADS_BAR = 1;
	Delay1TCY();			//Delay of 83.33nS (tAW)

	WR_BAR = 0;
	Delay1TCY();			//Delay of 83.33nS (tWR)
	Delay1TCY();			//Delay of 83.33nS (tWR)

	DataBus = WriteByte;

	WR_BAR = 1;
	Delay1TCY();			//Delay of 83.33nS

	EN_BAR = 1;
}