//Maxim Integrated Products DS1WM API Revision 1.2

#ifndef __DS1WM_H
#define __DS1WM_H

#define BYTE unsigned char
#define MaxNumberDevices 20		//Maximum number of devices on 1-Wire (this is user defined).

//PROTOTYPES
void CommandRegisterWrite(BYTE);
BYTE CommandRegisterRead();
void TransmitBufferWrite(BYTE);
BYTE ReceiveBufferRead();
BYTE InterruptRegisterRead();
void InterruptEnableRegisterWrite(BYTE);
BYTE InterruptEnableRegisterRead();
void ClockDivisorRegisterWrite(BYTE);
BYTE ClockDivisorRegisterRead();
void ControlRegisterWrite(BYTE);
BYTE ControlRegisterRead();
void WriteByte2Register(BYTE,BYTE);
BYTE ReadByteFromRegister(BYTE);
int OWReset();
BYTE OWWriteByte(BYTE);
BYTE OWWriteBytes(BYTE *, int);
BYTE OWReadByte();
void OWReadBytes(BYTE *,int);
void SetClockFrequency(BYTE);
int MatchROM(BYTE [][8],BYTE);
int OWSearch(BYTE [][8]);
int ROMCRCCheck(BYTE [][8],BYTE);

//** REGISTER DEFINITIONS *******************************************************
//Register Addresses
#define CommandRegister				0x00
#define TransmitBufferRegister		0x01
#define InterruptRegister			0x02
#define InterruptEnableRegister		0x03
#define ClockDivisorRegister		0x04
#define ControlRegister				0x05

//Interrupt Register Bit Locations (Used for logical ANDing)
#define	PD			0x01	//Presence Detect
#define	PDR			0x02	//Presence Detect Result
#define	TBE			0x04	//Transmit Buffer Empty
#define	TEMT		0x08	//Transmit Shift Register Empty
#define	RBF			0x10	//Receive Buffer Full
#define	RSRF		0x20	//Receive Shift Register Full
#define	OW_SHORT	0x40	//One Wire Short
#define	OW_LOW		0x80	//One Wire Low

//Command Register Bit Locations (Used for logical ANDing)
//NOTE: Only four bits are used in this register.
#define	OneWR		0x01	//1-Wire Reset (1WR)
#define	SRA			0x02	//Search ROM Accelerator
#define	TBE			0x04	//Force One Wire
#define	TEMT		0x08	//One Wire Input

//** PIN DEFINITIONS *******************************************************
//Data Bus: This bus comprises eight input/output lines. The bus provides bi-directional
//communications between the 1-Wire master and the CPU. Data, control words, and status
//information are transferred via this D7-D0 Data Bus.
//NOTE: If this port is changed, the TRISx statments in the functions need to change too.
//(INPUT/OUTPUT)
#define DataBus		PORTD	//8-Bit Bi-Directional Bus

//Register Select: Address signals connected to these three inputs select a register
//for the CPU to read from or write to during data transfer. A table of registers and
//their addresses is shown below. (OUTPUT)
//Register Addresses
//A2 A1 A0 Register
//0  0  0  Command Register (read/write)
//0  0  1  Transmit Buffer (write), Receive Buffer (read)
//0  1  0  Interrupt Register (read)
//0  1  1  Interrupt Enable Register (read/write)
//1  0  0  Clock Divisor Register (read/write)
//1  0  1  Control Register (read/write)
#define A0			PORTAbits.RA0
#define A1			PORTAbits.RA1
#define A2			PORTAbits.RA2

//Address Strobe: The positive edge of an active Address Strobe (ADS) signal latches
//the Register Select (A0, A1, A2) into an internal latch. Provided that setup and
//hold timings are observed, ADS may be tied low making the latch transparent.
//Pin Type: Output
#define ADS_BAR		PORTAbits.RA3

//Enable: When EN is low, the 1-Wire master is enabled; this signal acts as the device
//chip enable. This enables communication between the 1-Wire master and the CPU.
//Pin Type: Output
#define EN_BAR		PORTAbits.RA4

//Interrupt: This line goes to its active state whenever any one of the interrupt types
//has an active high condition and is enabled via the Interrupt Enable Register. The
//INTR signal is reset to an inactive state when the Interrupt Register is read.
//NOTE: Map INTR to a hardware interrupt pin if polling will not be used.
//Pin Type: Input
#define INTR		PORTAbits.RA5

//Master Reset: When this input is high, it clears all the registers and the control
//logic of the 1-Wire master, and sets INTR to its default inactive state, which is HIGH.
//Pin Type: Output
#define MR			PORTBbits.RB0

//Write: This pin drives the bus during a write cycle. When WR is low while the circuit
//is enabled, the CPU can write control words or data into the selected register. RD
//and WR should never be low simultaneously; if they are, WR takes precedence.
//Pin Type: Output
#define WR_BAR		PORTBbits.RB1

//Read: This pin drives the bus during a read cycle. When the circuit is enabled, the
//CPU can read status information or data from the selected register by driving RD
//low. RD and WR should never be low simultaneously; if they are, WR takes precedence.
//Pin Type: Output
#define RD_BAR		PORTBbits.RB2

#endif